@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.test.test_jelly -*-
-# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -54,8 +54,7 @@ Instance Method: s.center, where s is an instance of UserString.UserString::
 The C{set} builtin and the C{sets.Set} class are serialized to the same
 thing, and unserialized to C{set} if available, else to C{sets.Set}. It means
 that there's a possibility of type switching in the serialization process. The
-solution is to always use C{set} if possible, and only use C{sets.Set} under
-Python 2.3; this can be accomplished by using L{twisted.python.compat.set}.
+solution is to always use C{set}.
 
 The same rule applies for C{frozenset} and C{sets.ImmutableSet}.
 
@@ -66,6 +65,8 @@ The same rule applies for C{frozenset} and C{sets.ImmutableSet}.
 import pickle
 import types
 import warnings
+import decimal
+from functools import reduce
 from types import StringType
 from types import UnicodeType
 from types import IntType
@@ -85,16 +86,6 @@ import copy
 import datetime
 from types import BooleanType
 
-try:
-    import decimal
-except ImportError:
-    decimal = None
-
-try:
-    _set = set
-except NameError:
-    _set = None
-
 try:
     # Filter out deprecation warning for Python >= 2.6
     warnings.filterwarnings("ignore", category=DeprecationWarning,
@@ -104,8 +95,6 @@ finally:
     warnings.filters.pop()
 
 
-from new import instance
-from new import instancemethod
 from zope.interface import implements
 
 # Twisted Imports
@@ -113,17 +102,10 @@ from twisted.python.reflect import namedObject, qual
 from twisted.persisted.crefutil import NotKnown, _Tuple, _InstanceMethod
 from twisted.persisted.crefutil import _DictKeyAndValue, _Dereference
 from twisted.persisted.crefutil import _Container
-from twisted.python import runtime
 
 from twisted.spread.interfaces import IJellyable, IUnjellyable
 
-
-if runtime.platform.getType() == "java":
-    from org.python.core import PyStringMap
-    DictTypes = (DictionaryType, PyStringMap)
-else:
-    DictTypes = (DictionaryType,)
-
+DictTypes = (DictionaryType,)
 
 None_atom = "None"                  # N
 # code
@@ -153,20 +135,29 @@ unpersistable_atom = "unpersistable"# u
 unjellyableRegistry = {}
 unjellyableFactoryRegistry = {}
 
+_NO_STATE = object()
 
-
-def _newInstance(cls, state):
+def _newInstance(cls, state=_NO_STATE):
     """
     Make a new instance of a class without calling its __init__ method.
-    'state' will be used to update inst.__dict__ . Supports both new- and
-    old-style classes.
+    Supports both new- and old-style classes.
+
+    @param state: A C{dict} used to update C{inst.__dict__} or C{_NO_STATE}
+        to skip this part of initialization.
+
+    @return: A new instance of C{cls}.
     """
     if not isinstance(cls, types.ClassType):
         # new-style
         inst = cls.__new__(cls)
-        inst.__dict__.update(state) # Copy 'instance' behaviour
+
+        if state is not _NO_STATE:
+            inst.__dict__.update(state) # Copy 'instance' behaviour
     else:
-        inst = instance(cls, state)
+        if state is not _NO_STATE:
+            inst = InstanceType(cls, state)
+        else:
+            inst = InstanceType(cls)
     return inst
 
 
@@ -517,7 +508,7 @@ class _Jellier:
                                                    obj.microseconds)]
             elif objType is ClassType or issubclass(objType, type):
                 return ['class', qual(obj)]
-            elif decimal is not None and objType is decimal.Decimal:
+            elif objType is decimal.Decimal:
                 return self.jelly_decimal(obj)
             else:
                 preRef = self._checkMutable(obj)
@@ -533,11 +524,9 @@ class _Jellier:
                     sxp.append(dictionary_atom)
                     for key, val in obj.items():
                         sxp.append([self.jelly(key), self.jelly(val)])
-                elif (_set is not None and objType is set or
-                      objType is _sets.Set):
+                elif objType is set or objType is _sets.Set:
                     sxp.extend(self._jellyIterable(set_atom, obj))
-                elif (_set is not None and objType is frozenset or
-                      objType is _sets.ImmutableSet):
+                elif objType is frozenset or objType is _sets.ImmutableSet:
                     sxp.extend(self._jellyIterable(frozenset_atom, obj))
                 else:
                     className = qual(obj.__class__)
@@ -674,7 +663,7 @@ class _Unjellier:
             if not self.taster.isClassAllowed(clz):
                 raise InsecureJelly("Class %s not allowed." % jelType)
             if hasattr(clz, "__setstate__"):
-                ret = _newInstance(clz, {})
+                ret = _newInstance(clz)
                 state = self.unjelly(obj[1])
                 ret.__setstate__(state)
             else:
@@ -698,12 +687,8 @@ class _Unjellier:
 
     def _unjelly_decimal(self, exp):
         """
-        Unjelly decimal objects, if decimal is available. If not, return a
-        L{Unpersistable} object instead.
+        Unjelly decimal objects.
         """
-        if decimal is None:
-            return Unpersistable(
-                "Could not unpersist decimal: %s" % (exp[0] * (10**exp[1]),))
         value = exp[0]
         exponent = exp[1]
         if value < 0:
@@ -815,26 +800,16 @@ class _Unjellier:
 
     def _unjelly_set(self, lst):
         """
-        Unjelly set using either the C{set} builtin if available, or
-        C{sets.Set} as fallback.
+        Unjelly set using the C{set} builtin.
         """
-        if _set is not None:
-            containerType = set
-        else:
-            containerType = _sets.Set
-        return self._unjellySetOrFrozenset(lst, containerType)
+        return self._unjellySetOrFrozenset(lst, set)
 
 
     def _unjelly_frozenset(self, lst):
         """
-        Unjelly frozenset using either the C{frozenset} builtin if available,
-        or C{sets.ImmutableSet} as fallback.
+        Unjelly frozenset using the C{frozenset} builtin.
         """
-        if _set is not None:
-            containerType = frozenset
-        else:
-            containerType = _sets.ImmutableSet
-        return self._unjellySetOrFrozenset(lst, containerType)
+        return self._unjellySetOrFrozenset(lst, frozenset)
 
 
     def _unjelly_dictionary(self, lst):
@@ -927,9 +902,7 @@ class _Unjellier:
             elif isinstance(im_self, NotKnown):
                 im = _InstanceMethod(im_name, im_self, im_class)
             else:
-                im = instancemethod(im_class.__dict__[im_name],
-                                    im_self,
-                                    im_class)
+                im = MethodType(im_class.__dict__[im_name], im_self, im_class)
         else:
             raise TypeError('instance method changed')
         return im
@@ -950,6 +923,25 @@ class _DummyNewStyle(object):
     """
 
 
+def _newDummyLike(instance):
+    """
+    Create a new instance like C{instance}.
+
+    The new instance has the same class and instance dictionary as the given
+    instance.
+
+    @return: The new instance.
+    """
+    if isinstance(instance.__class__, type):
+        # New-style class
+        dummy = _DummyNewStyle()
+    else:
+        # Classic class
+        dummy = _Dummy()
+    dummy.__class__ = instance.__class__
+    dummy.__dict__ = instance.__dict__
+    return dummy
+
 
 #### Published Interface.
 
@@ -1023,8 +1015,7 @@ class SecurityOptions:
                              "NoneType": 1}
         if hasattr(types, 'UnicodeType'):
             self.allowedTypes['unicode'] = 1
-        if decimal is not None:
-            self.allowedTypes['decimal'] = 1
+        self.allowedTypes['decimal'] = 1
         self.allowedTypes['set'] = 1
         self.allowedTypes['frozenset'] = 1
         self.allowedModules = {}
